System.register(["tslib", "../../core/AtomBinder", "../../core/AtomLoader", "../../core/AtomUri", "../../services/NavigationService", "../../view-model/AtomWindowViewModel", "../core/AtomUI", "../styles/AtomFrameStyle", "./AtomControl"], function (_export, _context) {
  "use strict";

  var __awaiter, AtomBinder, AtomLoader, AtomUri, NavigationService, AtomWindowViewModel, AtomUI, AtomFrameStyle, AtomControl, AtomFrame;
  _export("AtomFrame", void 0);
  return {
    setters: [function (_tslib) {
      __awaiter = _tslib.__awaiter;
    }, function (_coreAtomBinder) {
      AtomBinder = _coreAtomBinder.AtomBinder;
    }, function (_coreAtomLoader) {
      AtomLoader = _coreAtomLoader.AtomLoader;
    }, function (_coreAtomUri) {
      AtomUri = _coreAtomUri.AtomUri;
    }, function (_servicesNavigationService) {
      NavigationService = _servicesNavigationService.NavigationService;
    }, function (_viewModelAtomWindowViewModel) {
      AtomWindowViewModel = _viewModelAtomWindowViewModel.AtomWindowViewModel;
    }, function (_coreAtomUI) {
      AtomUI = _coreAtomUI.AtomUI;
    }, function (_stylesAtomFrameStyle) {
      AtomFrameStyle = _stylesAtomFrameStyle.default;
    }, function (_AtomControl) {
      AtomControl = _AtomControl.AtomControl;
    }],
    execute: function () {
      _export("AtomFrame", AtomFrame = class AtomFrame extends AtomControl {
        get canGoBack() {
          return this.stack.length ? true : false;
        }
        get url() {
          return this.mUrl;
        }
        set url(value) {
          if (this.mUrl === value) {
            return;
          }
          if (value === undefined) {
            return;
          }
          this.mUrl = value;
          this.runAfterInit(() => {
            this.app.runAsync(() => this.loadForReturn(value === null ? null : new AtomUri(value), true));
          });
        }
        constructor(app, e) {
          super(app, e || document.createElement("section"));
        }
        onBackCommand() {
          return __awaiter(this, void 0, void 0, function* () {
            if (!this.stack.length) {
              console.warn(`FrameStack is empty !!`);
              return;
            }
            const ctrl = this.current;
            if (ctrl) {
              yield this.navigationService.remove(ctrl);
            }
          });
        }
        popStack(windowClosed) {
          if (!this.stack.length) {
            console.warn(`FrameStack is empty !!`);
            return;
          }
          const last = this.stack.pop();
          AtomBinder.refreshItems(this.stack);
          const old = this.current;
          this.current = last.page;
          this.current.element.style.display = "";
          if (old) {
            this.navigationService.remove(old).catch(e => console.log(e));
          }
          this.setUrl(last.url);
          if (this.saveScrollPosition) {
            setTimeout(() => {
              window.scrollTo(0, last.scrollY);
            }, 200);
          }
        }
        canChange() {
          const c = this.current;
          if (!c) {
            return Promise.resolve(true);
          }
          return this.navigationService.remove(c);
        }
        push(ctrl) {
          if (this.current) {
            if (this.keepStack) {
              this.current.element.style.display = "none";
              this.stack.add({
                url: this.current._$_url,
                page: this.current,
                scrollY: this.navigationService.screen.scrollTop
              });
            } else {
              if (this.current === ctrl) {
                return;
              }
              const c1 = this.current;
              const e1 = c1.element;
              if (e1) {
                this.navigationService.remove(c1);
              }
            }
          }
          const element = ctrl.element;
          const e = this.pagePresenter || this.element;
          e.appendChild(element);
          this.current = ctrl;
          if (this.saveScrollPosition) {
            window.scrollTo(0, 0);
          }
        }
        load(url, clearHistory) {
          return __awaiter(this, void 0, void 0, function* () {
            if (clearHistory) {
              if (!(yield this.canChange())) {
                return;
              }
            }
            const {
              view,
              disposables
            } = yield AtomLoader.loadView(url, this.app, true, () => new AtomWindowViewModel(this.app));
            const urlString = url.host ? url.toString() : url.pathAndQuery;
            view._$_url = urlString;
            this.push(view);
            const e = view.element;
            this.setUrl(urlString);
            disposables.add({
              dispose: () => {
                const closed = this.current === view;
                e.innerHTML = "";
                e.remove();
                this.popStack(closed);
              }
            });
            return view;
          });
        }
        toUpperCase(s) {
          return s.split("-").filter(t => t).map(t => t.substr(0, 1).toUpperCase() + t.substr(1)).join("");
        }
        setUrl(urlString) {
          this.mUrl = urlString;
          AtomBinder.refreshValue(this, "url");
          AtomBinder.refreshValue(this, "canGoBack");
        }
        loadForReturn(url, clearHistory) {
          return __awaiter(this, void 0, void 0, function* () {
            const hasHistory = this.keepStack;
            this.keepStack = !clearHistory;
            if (url === null) {
              if (hasHistory && clearHistory) {
                this.clearStack();
              }
              return;
            }
            const page = yield this.load(url, clearHistory);
            if (hasHistory) {
              if (clearHistory) {
                this.clearStack();
              }
            }
            try {
              return yield page.returnPromise;
            } catch (ex) {
              if ((ex.message ? ex.message : ex) === "cancelled") {
                console.warn(ex);
                return;
              }
              throw ex;
            }
          });
        }
        clearStack() {
          for (const iterator of this.stack) {
            const e = iterator.page.element;
            if (e) {
              iterator.page.dispose();
              e.innerHTML = "";
              e.remove();
            }
          }
          this.stack.clear();
        }
        preCreate() {
          this.name = null;
          this.stack = [];
          this.keepStack = false;
          this.current = null;
          this.currentDisposable = null;
          this.saveScrollPosition = false;
          this.navigationService = this.app.resolve(NavigationService);
          this.defaultControlStyle = AtomFrameStyle;
          this.pagePresenter = null;
          this.mUrl = null;
          AtomUI.assignID(this.element);
          this.runAfterInit(() => {
            this.setPrimitiveValue(this.element, "styleClass", this.controlStyle.name);
          });
          this.backCommand = () => this.app.runAsync(() => this.onBackCommand());
          const d = this.navigationService.registerNavigationHook((url, {
            target,
            clearHistory,
            cancelToken
          }) => {
            if (this.name) {
              if (target !== this.name) {
                return undefined;
              }
            } else {
              if (target !== "frame" && url.protocol !== "frame:") {
                return undefined;
              }
            }
            if (cancelToken) {
              cancelToken.registerForCancel(() => {
                this.backCommand();
              });
            }
            return this.loadForReturn(url, clearHistory);
          });
          this.registerDisposable(d);
        }
      });
    }
  };
});
//# sourceMappingURL=AtomFrame.js.map